package provide DS9 1.0

proc AnnulusDialog {frame id} {
    global marker
    global ds9

    # see if we already have a header window visible

    set w ".marker$id"
    set mb ".mb$id"

    if [winfo exist $w] {
	return
    }

    # Init common items

    InitMarkerDialog $frame $id
    PropMenuAnnulusDialog $frame $mb $id
    CoordMenuMarkerDialog $frame $mb $id CoordAnnulusCB

    # file menu items

    $mb.file add command -label "Apply" \
	-command "ApplyAnnulusDialog $frame $id"
    $mb.file add separator
    $mb.file add command -label "Close" \
	-command "CloseAnnulusDialog $frame $id"

    # annulus menu items

    set marker($frame,$id,radiusformat) pixels
    DistMenuMarkerDialog $frame $mb $id CoordAnnulusCB radiusformat
    set marker($frame,$id,method) dist
    AnnuliMenuMarkerDialog $frame $mb $id method
    $mb add cascade -label Radius -menu $mb.radiusformat
    $mb add cascade -label Method -menu $mb.method

    # annulus specific callbacks

    $frame marker $id callback edit EditAnnulusCB $frame
    $frame marker $id callback delete DeleteAnnulusDialog $frame

    # Dialog

    set type [string toupper "[$frame get marker $id type]"]
    toplevel $w -colormap $ds9(main)
    wm title $w $type
    wm iconname $w $type
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW "CloseAnnulusDialog $frame $id"

    $w configure -menu $mb

    frame $w.ref
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.ref -fill x
    pack $w.buttons -fill x -ipadx 4 -ipady 4

    frame $w.ref.left
    frame $w.ref.right -relief groove -borderwidth 2
    pack $w.ref.left $w.ref.right -side left -fill y

    frame $w.ref.left.top -relief groove -borderwidth 2
    frame $w.ref.left.bottom -relief groove -borderwidth 2
    pack $w.ref.left.top $w.ref.left.bottom -fill both -expand true

    # ID

    label $w.ref.left.top.idTitle -text "Id"
    label $w.ref.left.top.idValue -text "$id"

    # Text

    label $w.ref.left.top.textTitle -text "Text"
    entry $w.ref.left.top.textValue -textvariable marker($frame,$id,text) \
	-width 40 -relief sunken -bd 2

    grid $w.ref.left.top.idTitle -row 0 -column 0 -padx 4 -sticky w
    grid $w.ref.left.top.idValue -row 0 -column 1 -padx 4 -sticky w

    grid $w.ref.left.top.textTitle -row 1 -column 0 -padx 4 -sticky w
    grid $w.ref.left.top.textValue -row 1 -column 1 -padx 4 -sticky w \
	-columnspan 3

    # Center

    label $w.ref.left.bottom.centerTitle -text "Center"
    entry $w.ref.left.bottom.centerX -textvariable marker($frame,$id,x) \
	-width 13 -relief sunken -bd 2
    entry $w.ref.left.bottom.centerY -textvariable marker($frame,$id,y) \
	-width 13 -relief sunken -bd 2
    label $w.ref.left.bottom.centerCoord -relief groove -bd 2 -width 9 \
	-padx 4 -textvariable marker($frame,$id,coord)

    grid $w.ref.left.bottom.centerTitle -row 0 -column 0 -padx 4 -sticky w
    grid $w.ref.left.bottom.centerX -row 0 -column 1 -padx 4 -sticky w
    grid $w.ref.left.bottom.centerY -row 0 -column 2 -padx 4 -sticky w
    grid $w.ref.left.bottom.centerCoord -row 0 -column 3 -padx 4 -sticky w

    # Radius

    label $w.ref.left.bottom.innerTitle -text "Inner"
    label $w.ref.left.bottom.outerTitle -text "Outer"
    label $w.ref.left.bottom.radiusTitle -text "Radius"
    entry $w.ref.left.bottom.inner -textvariable marker($frame,$id,inner) \
	-width 13 -relief sunken -bd 2
    entry $w.ref.left.bottom.outer -textvariable marker($frame,$id,outer) \
	-width 13 -relief sunken -bd 2
    label $w.ref.left.bottom.radiusformat \
	-textvariable marker($frame,$id,radiusformat) \
	-relief groove -bd 2 -width 9 -padx 4
    label $w.ref.left.bottom.annuliTitle -text "Annuli"
    entry $w.ref.left.bottom.annuli -textvariable marker($frame,$id,annuli) \
	-width 13 -relief sunken -bd 2

    grid $w.ref.left.bottom.innerTitle -row 1 -column 1 -padx 4 -sticky w
    grid $w.ref.left.bottom.outerTitle -row 1 -column 2 -padx 4 -sticky w
    grid $w.ref.left.bottom.radiusTitle -row 2 -column 0 -padx 4 -sticky w
    grid $w.ref.left.bottom.inner -row 2 -column 1 -padx 4 -sticky w
    grid $w.ref.left.bottom.outer -row 2 -column 2 -padx 4 -sticky w
    grid $w.ref.left.bottom.radiusformat -row 2 -column 3 -padx 4 -sticky w
    grid $w.ref.left.bottom.annuliTitle -row 3 -column 0 -padx 4 -sticky w
    grid $w.ref.left.bottom.annuli -row 3 -column 1 -padx 4 -sticky w

    # Right

    frame $w.ref.right.title
    frame $w.ref.right.value
    pack $w.ref.right.title $w.ref.right.value -side top -padx 4 -pady 4

    label $w.ref.right.title.title -text "Radius"
    label $w.ref.right.title.radiusformat \
	-textvariable marker($frame,$id,radiusformat) \
	-relief groove -bd 2 -width 9 -padx 4

    grid $w.ref.right.title.title $w.ref.right.title.radiusformat \
	-padx 4 -sticky news

    set txtscr $w.ref.right.value.yscroll
    set marker($frame,$id,annulitxt) $w.ref.right.value.txt
    text $marker($frame,$id,annulitxt) -height 10 -width 15 -wrap none \
	-font courier -yscrollcommand "$txtscr set"
    scrollbar $txtscr -command [list $marker($frame,$id,annulitxt) yview] \
	-orient vertical

    grid $marker($frame,$id,annulitxt) $txtscr -sticky news

    # Buttons

    button $w.buttons.apply -text "Apply" \
	-command "ApplyAnnulusDialog $frame $id"
    button $w.buttons.generate -text "Generate" \
	-command "AnnulusUpdateRadius $frame $id"
    button $w.buttons.close -text "Close" \
	-command "CloseAnnulusDialog $frame $id"
    pack $w.buttons.apply $w.buttons.generate $w.buttons.close \
	-side left -padx 10 -expand true

    # Init annulus variables

    EditAnnulusCB $frame $id
}

proc PropMenuAnnulusDialog {frame mb id} {
    global menu
    global marker

    menu $mb.properties -tearoff 0 -selectcolor $menu(selectcolor)
    $mb.properties add checkbutton -label "Can Edit" \
	-variable marker($frame,$id,edit) \
	-command "PropertyMarkerDialog $frame $id edit"
    $mb.properties add checkbutton -label "Can Move" \
	-variable marker($frame,$id,move) \
	-command "PropertyMarkerDialog $frame $id move"
    $mb.properties add checkbutton -label "Can Delete" \
	-variable marker($frame,$id,delete) \
	-command "PropertyMarkerDialog $frame $id delete"
    $mb.properties add checkbutton -label "Include/Exclude" \
	-variable marker($frame,$id,include) \
	-command "PropertyMarkerDialog $frame $id include"
    $mb.properties add checkbutton -label "Fixed in Size" \
	-variable marker($frame,$id,fixed) \
	-command "PropertyMarkerDialog $frame $id fixed"
}

proc ApplyAnnulusDialog {frame id} {
    EditAnnulusDialog $frame $id
    ApplyMarkerDialog $frame $id
}

proc CloseAnnulusDialog {frame id} {
    global marker

    unset marker($frame,$id,inner)
    unset marker($frame,$id,outer)
    unset marker($frame,$id,annuli)
    unset marker($frame,$id,annulitxt)
    unset marker($frame,$id,radiusformat)
    unset marker($frame,$id,method)

    $frame marker $id delete callback edit EditAnnulusCB
    $frame marker $id delete callback delete DeleteAnnulusDialog
    DeleteMarkerCBs $frame $id

    DeleteMarkerDialog $frame $id
}

proc DeleteAnnulusDialog {frame id} {
    global marker

    unset marker($frame,$id,inner)
    unset marker($frame,$id,outer)
    unset marker($frame,$id,annuli)
    unset marker($frame,$id,annulitxt)
    unset marker($frame,$id,radiusformat)

    set mb ".mb$id"
    destroy $mb.radiusformat

    DeleteMarkerDialog $frame $id
}

proc EditAnnulusDialog {frame id} {
    global marker

    set levels ""
    regsub -all "\n" "[$marker($frame,$id,annulitxt) get 1.0 end]" " " levels
    # and trim any trailing spaces
    set levels [string trimright $levels " "]

    if {$levels != ""} {
	$frame marker $id annulus radius "\{$levels\}" \
	    $marker($frame,$id,radiusformat)
    }
}

proc EditAnnulusCB {frame id} {
    global marker

    set t \
       [$frame get marker $id annulus radius $marker($frame,$id,radiusformat)]

    set last [expr [llength $t]-1]
    set marker($frame,$id,inner) [lindex $t 0]
    set marker($frame,$id,outer) [lindex $t $last]
    set marker($frame,$id,annuli) $last

    $marker($frame,$id,annulitxt) delete 1.0 end
    $marker($frame,$id,annulitxt) insert end "$t"
}

proc CoordAnnulusCB {frame id} {
    MoveMarkerCB $frame $id
    EditAnnulusCB $frame $id
}

proc AnnulusUpdateRadius {frame id} {
    global marker

    $marker($frame,$id,annulitxt) delete 1.0 end

    set inner $marker($frame,$id,inner)
    set outer $marker($frame,$id,outer)
    set annuli $marker($frame,$id,annuli)

    if {($inner != "") && ($outer != "") && ($annuli != "")} {
	switch $marker($frame,$id,method) {
	    dist {
		for {set i 0} {$i<=$annuli} {incr i} {
		    $marker($frame,$id,annulitxt) insert end \
 		    "[expr ((($outer-$inner)/double($annuli))*$i)+$inner]\n"
		}
	    }

	    area {
		set pi 3.14159265358979323846
		set area [expr $pi*(($outer*$outer)-($inner*$inner))/$annuli]

		set r0 $inner
		$marker($frame,$id,annulitxt) insert end "$r0\n"
		for {set i 0} {$i<$annuli} {incr i} {
		    set r1 [expr sqrt(($area+($pi*$r0*$r0))/$pi)]
		    $marker($frame,$id,annulitxt) insert end \
			[format "%.4f\n" $r1]
		    set r0 $r1
		}
	    }
	}
    }
}

proc AnnulusDefaultDialog {} {
    global annulus
    global ed

    set w ".annulus"
    set title "Default Annulus"
    set ed(ok) 0
    set ed(inner) $annulus(inner)
    set ed(outer) $annulus(outer)
    set ed(annuli) $annulus(annuli)

    DialogCreate $w $title -borderwidth 2
    frame $w.ed  -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.ed $w.buttons -fill x -ipadx 4 -ipady 4

    label $w.ed.innerTitle -text "Inner"
    label $w.ed.outerTitle -text "Outer"
    label $w.ed.radiusTitle -text "Radius"
    entry $w.ed.inner -textvariable ed(inner) -width 10 \
	-relief sunken -bd 2
    entry $w.ed.outer -textvariable ed(outer) -width 10 \
	-relief sunken -bd 2
    label $w.ed.unit -text "pixels" -relief groove -width 8
    label $w.ed.annuliTitle -text "Annuli"
    entry $w.ed.annuli -textvariable ed(annuli) -width 10 \
	-relief sunken -bd 2
    
    grid $w.ed.innerTitle -row 0 -column 1 -padx 4 -sticky w
    grid $w.ed.outerTitle -row 0 -column 2 -padx 4 -sticky w
    grid $w.ed.radiusTitle -row 1 -column 0 -padx 4 -sticky w
    grid $w.ed.inner -row 1 -column 1 -padx 4 -sticky w
    grid $w.ed.outer -row 1 -column 2 -padx 4 -sticky w
    grid $w.ed.unit -row 1 -column 3 -padx 4 -sticky w
    grid $w.ed.annuliTitle -row 2 -column 0 -padx 4 -sticky w
    grid $w.ed.annuli -row 2 -column 1 -padx 4 -sticky w

    button $w.buttons.ok -text "OK" -default active \
	-command {set ed(ok) 1}
    button $w.buttons.cancel -text "Cancel" -command {set ed(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set ed(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    DialogCenter $w 
    $w.ed.inner select range 0 end
    DialogWait $w ed(ok) $w.ed.inner
    DialogDismiss $w

    if {$ed(ok)} {
	set annulus(inner) $ed(inner)
	set annulus(outer) $ed(outer)
	set annulus(annuli) $ed(annuli)
    }
    unset ed(inner)
    unset ed(outer)
    unset ed(annuli)
}
