package provide DS9 1.0

proc BoxAnnulusDialog {frame id} {
    global marker
    global ds9

    # see if we already have a header window visible

    set w ".marker$id"
    set mb ".mb$id"

    if [winfo exist $w] {
	return
    }

    # Init common items

    InitMarkerDialog $frame $id
    PropMenuBoxAnnulusDialog $frame $mb $id
    CoordMenuMarkerDialog $frame $mb $id CoordBoxAnnulusCB

    # file menu items

    $mb.file add command -label "Apply" \
	-command "ApplyBoxAnnulusDialog $frame $id"
    $mb.file add separator
    $mb.file add command -label "Close" \
	-command "CloseBoxAnnulusDialog $frame $id"

    # annulus menu items

    set marker($frame,$id,sizeformat) pixels
    DistMenuMarkerDialog $frame $mb $id CoordBoxAnnulusCB sizeformat
    set marker($frame,$id,method) dist
    AnnuliMenuMarkerDialog $frame $mb $id method
    $mb add cascade -label Size -menu $mb.sizeformat
    $mb add cascade -label Method -menu $mb.method

    # annulus specific callbacks

    $frame marker $id callback edit EditBoxAnnulusCB $frame
    $frame marker $id callback edit MoveMarkerCB $frame
    $frame marker $id callback rotate RotateBoxAnnulusCB $frame
    $frame marker $id callback delete DeleteBoxAnnulusDialog $frame

    # Dialog

    set type [string toupper "[$frame get marker $id type]"]
    toplevel $w -colormap $ds9(main)
    wm title $w $type
    wm iconname $w $type
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW "CloseBoxAnnulusDialog $frame $id"

    $w configure -menu $mb

    frame $w.ref
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.ref -fill x
    pack $w.buttons -fill x -ipadx 4 -ipady 4

    frame $w.ref.left
    frame $w.ref.right -relief groove -borderwidth 2
    pack $w.ref.left $w.ref.right -side left -fill y

    frame $w.ref.left.top -relief groove -borderwidth 2
    frame $w.ref.left.bottom -relief groove -borderwidth 2
    pack $w.ref.left.top $w.ref.left.bottom -fill both -expand true \
	-ipadx 4 -ipady 4

    # ID

    label $w.ref.left.top.idTitle -text "Id"
    label $w.ref.left.top.idValue -text "$id"

    # Text

    label $w.ref.left.top.textTitle -text "Text"
    entry $w.ref.left.top.textValue -textvariable marker($frame,$id,text) \
	-width 40 -relief sunken -bd 2

    grid $w.ref.left.top.idTitle -row 0 -column 0 -padx 4 -sticky w
    grid $w.ref.left.top.idValue -row 0 -column 1 -padx 4 -sticky w

    grid $w.ref.left.top.textTitle -row 1 -column 0 -padx 4 -sticky w
    grid $w.ref.left.top.textValue -row 1 -column 1 -padx 4 -sticky w \
	-columnspan 3

    # Center

    label $w.ref.left.bottom.centerTitle -text "Center"
    entry $w.ref.left.bottom.centerX -textvariable marker($frame,$id,x) \
	-width 13 -relief sunken -bd 2
    entry $w.ref.left.bottom.centerY -textvariable marker($frame,$id,y) \
	-width 13 -relief sunken -bd 2
    label $w.ref.left.bottom.centerCoord -relief groove -bd 2 -width 9 \
	-padx 4 -textvariable marker($frame,$id,coord)

    grid $w.ref.left.bottom.centerTitle -row 0 -column 0 -padx 4 -sticky w
    grid $w.ref.left.bottom.centerX -row 0 -column 1 -padx 4 -sticky w
    grid $w.ref.left.bottom.centerY -row 0 -column 2 -padx 4 -sticky w
    grid $w.ref.left.bottom.centerCoord -row 0 -column 3 -padx 4 -sticky w

    # Size

    label $w.ref.left.bottom.majorTitle -text "Width"
    label $w.ref.left.bottom.minorTitle -text "Height"

    label $w.ref.left.bottom.innerTitle -text "Outer Size"
    entry $w.ref.left.bottom.size1 -textvariable marker($frame,$id,size1) \
	-width 13 -relief sunken -bd 2
    entry $w.ref.left.bottom.size2 -textvariable marker($frame,$id,size2) \
	-width 13 -relief sunken -bd 2
    label $w.ref.left.bottom.sizeformat \
	-textvariable marker($frame,$id,sizeformat) \
	-relief groove -bd 2 -width 9 -padx 4

    label $w.ref.left.bottom.outerTitle -text "Inner Size"
    entry $w.ref.left.bottom.size3 -textvariable marker($frame,$id,size3) \
	-width 13 -relief sunken -bd 2

    label $w.ref.left.bottom.annuliTitle -text "Annuli"
    entry $w.ref.left.bottom.annuli -textvariable marker($frame,$id,annuli) \
	-width 13 -relief sunken -bd 2

    label $w.ref.left.bottom.angleTitle -text "Angle"
    entry $w.ref.left.bottom.angle -textvariable marker($frame,$id,angle) \
	-width 13 -relief sunken -bd 2
    label $w.ref.left.bottom.angleUnit -relief groove -bd 2 -width 9 \
	-padx 4 -text "degrees"

    grid $w.ref.left.bottom.majorTitle -row 1 -column 1 -padx 4 -sticky w
    grid $w.ref.left.bottom.minorTitle -row 1 -column 2 -padx 4 -sticky w

    grid $w.ref.left.bottom.innerTitle -row 2 -column 0 -padx 4 -sticky w
    grid $w.ref.left.bottom.size1 -row 2 -column 1 -padx 4 -sticky w
    grid $w.ref.left.bottom.size2 -row 2 -column 2 -padx 4 -sticky w
    grid $w.ref.left.bottom.sizeformat -row 2 -column 3 -padx 4 -sticky w

    grid $w.ref.left.bottom.outerTitle -row 3 -column 0 -padx 4 -sticky w
    grid $w.ref.left.bottom.size3 -row 3 -column 1 -padx 4 -sticky w

    grid $w.ref.left.bottom.annuliTitle -row 4 -column 0 -padx 4 -sticky w
    grid $w.ref.left.bottom.annuli -row 4 -column 1 -padx 4 -sticky w

    grid $w.ref.left.bottom.angleTitle -row 5 -column 0 -padx 4 -sticky w
    grid $w.ref.left.bottom.angle -row 5 -column 1 -padx 4 -sticky w
    grid $w.ref.left.bottom.angleUnit -row 5 -column 3 -padx 4 -sticky w

    # Right

    frame $w.ref.right.title
    frame $w.ref.right.value
    pack $w.ref.right.title $w.ref.right.value -side top -padx 4 -pady 4

    label $w.ref.right.title.title -text "Annuli"
    label $w.ref.right.title.sizeformat \
	-textvariable marker($frame,$id,sizeformat) \
	-relief groove -bd 2 -width 9 -padx 4

    grid $w.ref.right.title.title $w.ref.right.title.sizeformat \
	-padx 4 -sticky news

    set txtscr $w.ref.right.value.yscroll
    set marker($frame,$id,annulitxt) $w.ref.right.value.txt
    text $marker($frame,$id,annulitxt) -height 10 -width 15 -wrap none \
	-font courier -yscrollcommand "$txtscr set"
    scrollbar $txtscr -command [list $marker($frame,$id,annulitxt) yview] \
	-orient vertical

    grid $marker($frame,$id,annulitxt) $txtscr -sticky news

    # Buttons

    button $w.buttons.apply -text "Apply" \
	-command "ApplyBoxAnnulusDialog $frame $id"
    button $w.buttons.generate -text "Generate" \
	-command "BoxAnnulusUpdateSize $frame $id"
    button $w.buttons.close -text "Close" \
	-command "CloseBoxAnnulusDialog $frame $id"
    pack $w.buttons.apply $w.buttons.generate $w.buttons.close \
	-side left -padx 10 -expand true

    # Init annulus variables

    EditBoxAnnulusCB $frame $id
    RotateBoxAnnulusCB $frame $id
}

proc PropMenuBoxAnnulusDialog {frame mb id} {
    global menu
    global marker

    menu $mb.properties -tearoff 0 -selectcolor $menu(selectcolor)
    $mb.properties add checkbutton -label "Can Edit" \
	-variable marker($frame,$id,edit) \
	-command "PropertyMarkerDialog $frame $id edit"
    $mb.properties add checkbutton -label "Can Move" \
	-variable marker($frame,$id,move) \
	-command "PropertyMarkerDialog $frame $id move"
    $mb.properties add checkbutton -label "Can Delete" \
	-variable marker($frame,$id,delete) \
	-command "PropertyMarkerDialog $frame $id delete"
    $mb.properties add checkbutton -label "Include/Exclude" \
	-variable marker($frame,$id,include) \
	-command "PropertyMarkerDialog $frame $id include"
    $mb.properties add checkbutton -label "Fixed in Size" \
	-variable marker($frame,$id,fixed) \
	-command "PropertyMarkerDialog $frame $id fixed"
}

proc ApplyBoxAnnulusDialog {frame id} {
    # do this first, since editbox will do a MoveMarkerCB
    ApplyMarkerDialog $frame $id

    EditBoxAnnulusDialog $frame $id
    RotateBoxAnnulusDialog $frame $id
}

proc CloseBoxAnnulusDialog {frame id} {
    global marker

    unset marker($frame,$id,size1)
    unset marker($frame,$id,size2)
    unset marker($frame,$id,size3)
    unset marker($frame,$id,annuli)
    unset marker($frame,$id,annulitxt)
    unset marker($frame,$id,sizeformat)
    unset marker($frame,$id,angle)

    $frame marker $id delete callback edit EditBoxAnnulusCB
    $frame marker $id delete callback edit MoveMarkerCB
    $frame marker $id delete callback rotate RotateBoxAnnulusCB
    $frame marker $id delete callback delete DeleteBoxAnnulusDialog
    DeleteMarkerCBs $frame $id

    DeleteMarkerDialog $frame $id
}

proc DeleteBoxAnnulusDialog {frame id} {
    global marker

    unset marker($frame,$id,size1)
    unset marker($frame,$id,size2)
    unset marker($frame,$id,size3)
    unset marker($frame,$id,annuli)
    unset marker($frame,$id,annulitxt)
    unset marker($frame,$id,sizeformat)
    unset marker($frame,$id,angle)

    set mb ".mb$id"
    destroy $mb.sizeformat

    DeleteMarkerDialog $frame $id
}

proc EditBoxAnnulusDialog {frame id} {
    global marker

    set levels ""
    regsub -all "\n" "[$marker($frame,$id,annulitxt) get 1.0 end]" " " levels
    # and trim any trailing spaces
    set levels [string trimright $levels " "]

    if {$levels != ""} {
	$frame marker $id box annulus size "\{$levels\}" \
	    $marker($frame,$id,sizeformat)
    }
}

proc RotateBoxAnnulusDialog {frame id} {
    global marker

    $frame marker $id box annulus angle $marker($frame,$id,angle)
}

proc EditBoxAnnulusCB {frame id} {
    global marker

    set t [$frame get marker $id box annulus size \
	       $marker($frame,$id,sizeformat)]

    set last [llength $t]
    set marker($frame,$id,annuli) [expr $last/2-1]
    set marker($frame,$id,size1) [lindex $t [expr $last-2]]
    set marker($frame,$id,size2) [lindex $t [expr $last-1]]
    set marker($frame,$id,size3) [lindex $t 0]

    $marker($frame,$id,annulitxt) delete 1.0 end
    $marker($frame,$id,annulitxt) insert end "$t"
}

proc RotateBoxAnnulusCB {frame id} {
    global marker

    set marker($frame,$id,angle) [$frame get marker $id box annulus angle]
}

proc CoordBoxAnnulusCB {frame id} {
    MoveMarkerCB $frame $id
    EditBoxAnnulusCB $frame $id
}

proc BoxAnnulusUpdateSize {frame id} {
    global marker

    $marker($frame,$id,annulitxt) delete 1.0 end

    set size1 $marker($frame,$id,size1)
    set size2 $marker($frame,$id,size2)
    set size3 $marker($frame,$id,size3)
    set annuli $marker($frame,$id,annuli)

    if {($size1 != "") && ($size2 != "") && \
        ($size3 != "") && ($annuli != "")} {

	if {$size1<=0} {
	    set size1 1
	}
	if {$size2<=0} {
	    set size2 1
	}

	switch $marker($frame,$id,method) {
	    dist {
		for {set i 0} {$i<=$annuli} {incr i} {
		    set major [expr ((($size1-$size3)/$annuli)*$i)+$size3]
		    set minor [expr $major*$size2/$size1]
		    $marker($frame,$id,annulitxt) insert end "$major $minor\n"
		}
	    }

	    area {
		set r [expr double($size2)/$size1]
		set area [expr (($size1*$size2)-($size3*$size3*$r)) \
			      /$annuli]

		set major0 $size3
		set minor0 [expr $size3*$r]
		$marker($frame,$id,annulitxt) insert end "$major0 $minor0\n"
		for {set i 0} {$i<$annuli} {incr i} {
		    set major1 [expr sqrt(($area+($major0*$minor0))/$r)]
		    set minor1 [expr $major1*$r]
		    $marker($frame,$id,annulitxt) insert end \
			[format "%.4f %.4f\n" $major1 $minor1]
		    set major0 $major1
		    set minor0 $minor1
		}
	    }
	}
    }
}

proc BoxAnnulusDefaultDialog {} {
    global boxannulus
    global ed

    set w ".boxannulus"
    set title "Default Box Annulus"
    set ed(ok) 0
    set ed(size1) $boxannulus(size1)
    set ed(size2) $boxannulus(size2)
    set ed(size3) $boxannulus(size3)
    set ed(annuli) $boxannulus(annuli)

    DialogCreate $w $title -borderwidth 2
    frame $w.ed  -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.ed $w.buttons -fill x -ipadx 4 -ipady 4

    label $w.ed.majorTitle -text "Major"
    label $w.ed.minorTitle -text "Minor"

    label $w.ed.innerTitle -text "Inner Size"
    entry $w.ed.size1 -textvariable ed(size1) -width 10 \
	-relief sunken -bd 2
    entry $w.ed.size2 -textvariable ed(size2) -width 10 \
	-relief sunken -bd 2
    label $w.ed.unit -text "pixels" -relief groove -width 8

    label $w.ed.outerTitle -text "Outer Size"
    entry $w.ed.size3 -textvariable ed(size3) -width 10 \
	-relief sunken -bd 2

    label $w.ed.annuliTitle -text "Annuli"
    entry $w.ed.annuli -textvariable ed(annuli) -width 10 \
	-relief sunken -bd 2
    
    grid $w.ed.majorTitle -row 0 -column 1 -padx 4 -sticky w
    grid $w.ed.minorTitle -row 0 -column 2 -padx 4 -sticky w

    grid $w.ed.innerTitle -row 1 -column 0 -padx 4 -sticky w
    grid $w.ed.size1 -row 1 -column 1 -padx 4 -sticky w
    grid $w.ed.size2 -row 1 -column 2 -padx 4 -sticky w
    grid $w.ed.unit -row 1 -column 3 -padx 4 -sticky w

    grid $w.ed.outerTitle -row 2 -column 0 -padx 4 -sticky w
    grid $w.ed.size3 -row 2 -column 1 -padx 4 -sticky w

    grid $w.ed.annuliTitle -row 3 -column 0 -padx 4 -sticky w
    grid $w.ed.annuli -row 3 -column 1 -padx 4 -sticky w

    button $w.buttons.ok -text "OK" -default active \
	-command {set ed(ok) 1}
    button $w.buttons.cancel -text "Cancel" -command {set ed(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set ed(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    DialogCenter $w 
    $w.ed.size1 select range 0 end
    DialogWait $w ed(ok) $w.ed.size1
    DialogDismiss $w

    if {$ed(ok)} {
	set boxannulus(size1) $ed(size1)
	set boxannulus(size2) $ed(size2)
	set boxannulus(size3) $ed(size3)
	set boxannulus(annuli) $ed(annuli)
    }
    unset ed(size1)
    unset ed(size2)
    unset ed(size3)
    unset ed(annuli)
}
