package provide DS9 1.0

proc ButtonButton {button text cmd} {
    global tcl_platform
    if {$tcl_platform(platform)!= "windows"} {
	set padx 2
	set pady 2
    } else {
	set padx 0
	set pady 0
    }
    button $button \
	-text $text \
	-command $cmd \
	-takefocus 0 \
	-bd 1 \
	-height 1 \
	-padx $padx -pady $pady

    pack $button -side left -fill x -expand true
}

proc RadioButton {button text var value cmd} {
    global tcl_platform
    if {$tcl_platform(platform)!= "windows"} {
	set padx 2
	set pady 3
    } else {
	set padx 0
	set pady 0
    }
    radiobutton $button \
	-text $text \
	-variable $var \
	-value $value \
	-command $cmd \
	-indicatoron false \
	-selectcolor "" \
	-takefocus 0 \
	-bd 1 \
	-height 1 \
	-padx $padx -pady $pady

    pack $button -side left -fill x -expand true
}

proc CheckButton {button text var cmd} {
    global tcl_platform
    if {$tcl_platform(platform)!= "windows"} {
	set padx 2
	set pady 3
    } else {
	set padx 0
	set pady 0
    }
    checkbutton $button \
	-text $text \
	-variable $var \
	-command $cmd \
	-indicatoron false \
	-selectcolor "" \
	-takefocus 0 \
	-bd 1 \
	-height 1 \
	-padx $padx -pady $pady

    pack $button -side left -fill x -expand true
}

proc CreateButtons {} {
    global buttons

    set buttons(name) [frame .ds9.buttons]

    # Major buttons

    frame $buttons(name).major
    pack $buttons(name).major -side top -fill x -expand true

    RadioButton $buttons(name).major.file File buttons(majorCurrent) \
	$buttons(name).file MajorButton
    RadioButton $buttons(name).major.frame Frame buttons(majorCurrent) \
	$buttons(name).frame MajorButton
    RadioButton $buttons(name).major.bin Bin buttons(majorCurrent) \
	$buttons(name).bin MajorButton
    RadioButton $buttons(name).major.zoom Zoom buttons(majorCurrent) \
	$buttons(name).zoom MajorButton
    RadioButton $buttons(name).major.scale Scale buttons(majorCurrent) \
	$buttons(name).scale MajorButton
    RadioButton $buttons(name).major.color1 Color buttons(majorCurrent) \
	$buttons(name).color1 MajorButton
    RadioButton $buttons(name).major.region Region buttons(majorCurrent) \
	$buttons(name).region1 MajorButton

    FileButtons
    FrameButtons
    BinButtons
    ZoomButtons
    ScaleButtons
    ColorButtons
    RegionButtons

    set buttons(majorPrev) ""
    set buttons(majorCurrent) $buttons(name).file
    MajorButton
}

proc MajorButton {} {
    global buttons

    pack forget $buttons(majorPrev)
    pack $buttons(majorCurrent) -side top -fill x -expand true
    set buttons(majorPrev) $buttons(majorCurrent)
}

proc FileButtons {} {
    global buttons

    frame $buttons(name).file

    ButtonButton $buttons(name).file.open open OpenFile
    ButtonButton $buttons(name).file.save save SaveFits
    ButtonButton $buttons(name).file.header header DisplayHeaderMenu
    ButtonButton $buttons(name).file.print print Print
    ButtonButton $buttons(name).file.exit exit QuitDS9
}

proc FrameButtons {} {
    global buttons
    global ds9

    frame $buttons(name).frame

    ButtonButton $buttons(name).frame.new new CreateFrame
    ButtonButton $buttons(name).frame.delete delete DeleteFrameMenu
    ButtonButton $buttons(name).frame.reset reset ResetCurrentFrame
    ButtonButton $buttons(name).frame.clear clear ClearCurrentFrame
    RadioButton $buttons(name).frame.single single ds9(displaymode) \
	single DisplayMode
    RadioButton $buttons(name).frame.tile tile ds9(displaymode) \
	tile DisplayMode
    RadioButton $buttons(name).frame.blink blink ds9(displaymode) \
	blink DisplayMode
    ButtonButton $buttons(name).frame.first first FirstFrame
    ButtonButton $buttons(name).frame.prev prev PrevFrame
    ButtonButton $buttons(name).frame.next next NextFrame
    ButtonButton $buttons(name).frame.last last LastFrame
}

proc BinButtons {} {
    global buttons

    frame $buttons(name).bin

    ButtonButton $buttons(name).bin.in in "Bin .5"
    ButtonButton $buttons(name).bin.out out "Bin 2"
    RadioButton $buttons(name).bin.b1 "block 1" \
	bin(factor) 1 ChangeBinFactor
    RadioButton $buttons(name).bin.b2 "block 2" \
	bin(factor) 2 ChangeBinFactor
    RadioButton $buttons(name).bin.b4 "block 4" \
	bin(factor) 4 ChangeBinFactor
    RadioButton $buttons(name).bin.b8 "block 8" \
	bin(factor) 8 ChangeBinFactor
    RadioButton $buttons(name).bin.b16 "block 16" \
	bin(factor) 16 ChangeBinFactor
    RadioButton $buttons(name).bin.b32 "block 32" \
	bin(factor) 32 ChangeBinFactor
    RadioButton $buttons(name).bin.b64 "block 64" \
	bin(factor) 64 ChangeBinFactor
    RadioButton $buttons(name).bin.b128 "block 128" \
	bin(factor) 128 ChangeBinFactor
}

proc ZoomButtons {} {
    global buttons
    global wcs

    frame $buttons(name).zoom

    ButtonButton $buttons(name).zoom.in "in" "Zoom 2"
    ButtonButton $buttons(name).zoom.out "out" "Zoom .5"
    ButtonButton $buttons(name).zoom.center "center" CenterCurrentFrame

    RadioButton $buttons(name).zoom.none none current(orient) none ChangeOrient
    RadioButton $buttons(name).zoom.x x current(orient) x ChangeOrient
    RadioButton $buttons(name).zoom.y y current(orient) y ChangeOrient
    RadioButton $buttons(name).zoom.xy xy current(orient) xy ChangeOrient

    CheckButton $buttons(name).zoom.align "align wcs" wcs(align) AlignWCSFrame
}

proc ScaleButtons {} {
    global buttons
    global scale

    frame $buttons(name).scale

    RadioButton $buttons(name).scale.minmax minmax scale(mode) minmax ScaleMode
    RadioButton $buttons(name).scale.zscale zscale scale(mode) zscale ScaleMode

    RadioButton $buttons(name).scale.linear linear \
	current(scale) linear ChangeColorScale
    RadioButton $buttons(name).scale.log log \
	current(scale) log ChangeColorScale
    RadioButton $buttons(name).scale.squared squared \
	current(scale) squared ChangeColorScale
    RadioButton $buttons(name).scale.sqrt sqrt \
	current(scale) sqrt ChangeColorScale
}

proc ColorButtons {} {
    global buttons
    global colorbar

    set ids [colorbar list id]

    frame $buttons(name).color1

    CheckButton $buttons(name).color1.invert invert \
	colorbar(invert) InvertColorbar
    for {set i 0} {$i<9} {incr i} {
	set id [lindex $ids $i]
	set title [colorbar get name $id]
	set name [string tolower $title]
	RadioButton $buttons(name).color1.$name $name \
	    colorbar(map) $title "ChangeColormapID $id"
    }
    RadioButton $buttons(name).color1.cmap cmap buttons(majorCurrent) \
	$buttons(name).color2 MajorButton

    frame $buttons(name).color2

    CheckButton $buttons(name).color2.invert invert \
	colorbar(invert) InvertColorbar
    for {set i 9} {$i<17} {incr i} {
	set id [lindex $ids $i]
	set title [colorbar get name $id]
	set name [string tolower $title]
	RadioButton $buttons(name).color2.$name $name \
	    colorbar(map) $title "ChangeColormapID $id"
    }
    RadioButton $buttons(name).color2.cmap cmap buttons(majorCurrent) \
	$buttons(name).color1 MajorButton
}

proc RegionButtons {} {
    global buttons

    frame $buttons(name).region1

    RadioButton $buttons(name).region1.circle circle marker(shape) circle ""
    RadioButton $buttons(name).region1.ellipse ellipse marker(shape) ellipse ""
    RadioButton $buttons(name).region1.box box marker(shape) box ""
    RadioButton $buttons(name).region1.polygon polygon marker(shape) polygon ""
    RadioButton $buttons(name).region1.line line marker(shape) line ""
    RadioButton $buttons(name).region1.text text marker(shape) text ""
    RadioButton $buttons(name).region1.ruler ruler marker(shape) ruler ""
    RadioButton $buttons(name).region1.point point \
	marker(shape) "circle point" ""

    RadioButton $buttons(name).region1.region region buttons(majorCurrent) \
	$buttons(name).region2 MajorButton

    frame $buttons(name).region2

    ButtonButton $buttons(name).region2.info info InfoMarker
    ButtonButton $buttons(name).region2.front front FrontMarker
    ButtonButton $buttons(name).region2.back back BackMarker
    ButtonButton $buttons(name).region2.all "sel all" SelectAllMarker
    ButtonButton $buttons(name).region2.none "sel none" UnselectAllMarker
    ButtonButton $buttons(name).region2.delete "del all" DeleteAllMarker
    ButtonButton $buttons(name).region2.load load OpenMarkerFile
    ButtonButton $buttons(name).region2.save save SaveMarker
    ButtonButton $buttons(name).region2.list list DisplayMarkerList

    RadioButton $buttons(name).region2.region region buttons(majorCurrent) \
	$buttons(name).region1 MajorButton
    
}



