package provide DS9 1.0

# Print procs

proc Print {} {
    global tcl_platform

    if {[PrintDialog ps]} {
	if {$tcl_platform(platform)!= "windows"} {
	    set w .status
	    DisplayStatusMsg $w "Printing, please wait..."
	    if [catch {PostScript} printError] {
		Error "A printing error has occured: $printError"
	    }
	    DestroyStatusMsg $w
	} else {
	    if [catch {PostScript} printError] {
		Error "A printing error has occured: $printError"
	    }
	}
    }
}

proc PostScript {} {
    global ps
    global canvas
    global ds9

    # Mode
    #    set options "-colormode $ps(color)"
    set options " "

    # Orientation

    switch $ps(orient) {
	portrait {set options "$options -rotate false"}
	landscape {set options "$options -rotate true"}
    }

    # Page size

    if {$ps(scale) == "scaled"} {
	switch $ps(size) {
	    letter {set options [PSPageSize 4.25i 5.5i 7.5i 10.i $options]}
	    legal {set options [PSPageSize 4.25i 7.i 7.5i 13.i $options]}
	    tabloid {set options [PSPageSize 5.5i 8.5i 10.i 16.i $options]}
	    poster {set options [PSPageSize 18.i 24.i 35.i 47.i $options]}
	    a4 {set options [PSPageSize 105m 148.5m 185m 272m $options]}
	    other {
		if {$ps(width) != "" && $ps(height) != ""} {
		    set x [expr double($ps(width))/2.]
		    set y [expr double($ps(height))/2.]
		    set w [expr $ps(width)-1.]
		    set h [expr $ps(height)-1.]
		    set options [PSPageSize [append x i] [append y i] \
				     [append w i] [append h i] $options]
		}
	    }
	    othermm {
		if {$ps(width) != "" && $ps(height) != ""} {
		    set x [expr double($ps(width))/2.]
		    set y [expr double($ps(height))/2.]
		    set w [expr $ps(width)-1.]
		    set h [expr $ps(height)-1.]
		    set options [PSPageSize [append x m] [append y m] \
				     [append w m] [append h m] $options]
		}
	    }
	}
    }

    # Printer vs File
    
    set channel ""
    if {$ps(dest) == "file" && $ps(fileName) != ""} {
	set options "$options -file $ps(fileName)"
    } else {
	set channel [open "| $ps(cmd)" w]
	set options "$options -channel $channel"
    }

    # set frame specific postscript options

    foreach f $ds9(frames) {
	$f postscript level $ps(level)
	$f postscript colorspace $ps(color)
	$f postscript interpolate $ps(interpolate)
	$f postscript resolution $ps(resolution)
    }

    # now invoke canvas postscript command

    if [catch {eval $canvas(name) postscript $options} psError] {
	Error "A postscript generation error has occured: $psError"
    }

    if {$channel != ""} {
	close $channel
    }
}

proc PSPageSize {x y w h options} {
    global ps
    global canvas

    set width [winfo width $canvas(name)]
    set height [winfo height $canvas(name)]
    set wo [string trim $w "icmp"]
    set ho [string trim $h "icmp"]

    switch $ps(orient) {
	portrait {
	    if {[expr double($wo)/$width] < [expr double($ho)/$height]} {
		set options "$options -pagewidth $w"
	    } else {
		set options "$options -pageheight $h"
	    }
	}
	landscape {
	    if {[expr double($wo)/$width] > [expr double($ho)/$height]} {
		set options "$options -pageheight $w"
	    } else {
		set options "$options -pagewidth $h"
	    }
	}
    }

    set options "$options -pagex $x -pagey $y"
    return $options
}

# Print Dialog procs

proc PrintDialog {which} {
    global print
    global ps
    global menu

    set print(dest) $ps(dest)
    set print(cmd) $ps(cmd)
    set print(fileName) $ps(fileName)
    set print(color) $ps(color)
    set print(level) $ps(level)
    set print(interpolate) $ps(interpolate)
    set print(resolution) $ps(resolution)

    set w ".print"

    set print(ok) 0

    DialogCreate $w "Print" -borderwidth 2
    frame $w.print -relief groove -borderwidth 2
    frame $w.options -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2

    if {$which == "ps"} {
	pack $w.print $w.options $w.buttons -fill x -ipadx 4 -ipady 4
    } else {
	pack $w.print $w.buttons -fill x -ipadx 4 -ipady 4
    }

    # Print

    label $w.print.title1 -text "Print To:"
    radiobutton $w.print.printer -text "Printer" -variable print(dest) \
	-value printer -selectcolor $menu(selectcolor)
    radiobutton $w.print.file -text "File" -variable print(dest) \
	-value file -selectcolor $menu(selectcolor)

    label $w.print.title2 -text "Print Command:"
    entry $w.print.cmd -textvariable print(cmd) -width 30 \
	-relief sunken -bd 2

    label $w.print.title3 -text "File Name:"
    entry $w.print.name -textvariable print(fileName) -width 30 \
	-relief sunken -bd 2

    button $w.print.browse -text "Browse" -command "PrintBrowse $w"

    grid rowconfigure $w.print 0 -pad 4
    grid rowconfigure $w.print 1 -pad 4
    grid rowconfigure $w.print 2 -pad 4

    global tcl_platform
    if {$tcl_platform(platform)!= "windows"} {
	grid $w.print.title1 -row 0 -column 0 -padx 4 -sticky w
	grid $w.print.printer -row 0 -column 1 -padx 4 -sticky w
	grid $w.print.file -row 0 -column 2 -padx 4 -sticky w

	grid $w.print.title2 -row 1 -column 0 -padx 4 -sticky w
	grid $w.print.cmd -row 1 -column 1 -columnspan 2 -padx 4 -sticky w
    }
    grid $w.print.title3 -row 2 -column 0 -padx 4 -sticky w
    grid $w.print.name -row 2 -column 1 -columnspan 2 -padx 4 -sticky w

    grid $w.print.browse -row 2 -column 3 -padx 4

    # Options

    label $w.options.title2 -text "Print:"
    radiobutton $w.options.rgb -text "RGB Color" -variable print(color) \
	-value rgb -selectcolor $menu(selectcolor)
    radiobutton $w.options.cmyk -text "CMYK Color" -variable print(color) \
	-value cmyk -selectcolor $menu(selectcolor)
    radiobutton $w.options.gray -text "Grayscale" -variable print(color) \
	-value gray -selectcolor $menu(selectcolor)

    label $w.options.title4 -text "Postscript:"
    radiobutton $w.options.level1 -text "Level 1" -variable print(level) \
	-value 1 -selectcolor $menu(selectcolor) -command "PSLevel $w"
    radiobutton $w.options.level2 -text "Level 2" -variable print(level) \
	-value 2 -selectcolor $menu(selectcolor) -command "PSLevel $w"

    checkbutton $w.options.interpolate -text "Interpolate" \
	-variable print(interpolate) -selectcolor $menu(selectcolor)

    label $w.options.title5 -text "Resolution:"

    frame $w.options.resolution
    label $w.options.resolution.title -text "dpi"
    menubutton $w.options.resolution.button -relief raised \
	-menu $w.options.resolution.button.menu \
	-textvariable print(resolution)
    menu $w.options.resolution.button.menu \
	-tearoff $menu(tearoff) -selectcolor $menu(selectcolor)
    $w.options.resolution.button.menu add radiobutton -label "53" \
	-variable print(resolution) -value 53
    $w.options.resolution.button.menu add radiobutton -label "72" \
	-variable print(resolution) -value 72
    $w.options.resolution.button.menu add radiobutton -label "75" \
	-variable print(resolution) -value 75
    $w.options.resolution.button.menu add radiobutton -label "150" \
	-variable print(resolution) -value 150
    $w.options.resolution.button.menu add radiobutton -label "300" \
	-variable print(resolution) -value 300
    $w.options.resolution.button.menu add radiobutton -label "600" \
	-variable print(resolution) -value 600
    pack $w.options.resolution.button $w.options.resolution.title \
	-side left

    grid rowconfigure $w.options 0 -pad 4
    grid rowconfigure $w.options 1 -pad 4
    grid rowconfigure $w.options 2 -pad 4

    grid $w.options.title2 -row 0 -column 0 -padx 4 -sticky w
    grid $w.options.rgb -row 0 -column 1 -padx 4 -sticky w
    grid $w.options.cmyk -row 0 -column 2 -padx 4 -sticky w
    grid $w.options.gray -row 0 -column 3 -padx 4 -sticky w

    grid $w.options.title4 -row 1 -column 0 -padx 4 -sticky w
    grid $w.options.level2 -row 1 -column 1 -padx 4 -sticky w
    grid $w.options.level1 -row 1 -column 2 -padx 4 -sticky w

    grid $w.options.interpolate -row 2 -column 1 -padx 4 -sticky w

    grid $w.options.title5 -row 3 -column 0 -padx 4 -sticky w
    grid $w.options.resolution -row 3 -column 1 -padx 4 -sticky w

    # Buttons

    button $w.buttons.ok -text "OK" -default active \
	-command {set print(ok) 1}
    button $w.buttons.cancel -text "Cancel" -command {set print(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set print(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    PSLevel $w

    DialogCenter $w
    DialogWait $w print(ok) $w.buttons.ok
    DialogDismiss $w

    if {$print(ok)} {
	set ps(dest) $print(dest)
	set ps(cmd) $print(cmd)
	set ps(fileName) $print(fileName)
	set ps(color) $print(color)
	set ps(level) $print(level)
	set ps(interpolate) $print(interpolate)
	set ps(resolution) $print(resolution)
    }

    return $print(ok)
}

proc PSLevel {w} {
    global print
    switch -- $print(level) {
	1 {
	    $w.options.interpolate configure -state disabled
	    $w.options.cmyk configure -state disabled
	    if {$print(color)=="cmyk"} {
		set print(color) rgb
	    }
	}
	2 {
	    $w.options.interpolate configure -state normal
	    $w.options.cmyk configure -state normal
	}
    }
}

proc PrintBrowse {w} {
    global print

    set init ""
    set fileName [SaveFileDialog $w init]

    if {$fileName != ""} {
	set print(fileName) $fileName
    }
}

proc PageDialog {} {
    global page
    global ps
    global menu

    set page(orient) $ps(orient)
    set page(scale) $ps(scale)
    set page(size) $ps(size)
    set page(width) $ps(width)
    set page(height) $ps(height)

    set w ".page"

    set page(ok) 0

    DialogCreate $w "Page Setup" -borderwidth 2
    frame $w.options -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.options $w.buttons -fill x -ipadx 4 -ipady 4

    # Options

    label $w.options.title1 -text "Orientation:"
    radiobutton $w.options.portrait -text "Portrait" \
	-variable page(orient) \
	-value portrait -selectcolor $menu(selectcolor)
    radiobutton $w.options.landscape -text "Landscape" \
	-variable page(orient) \
	-value landscape -selectcolor $menu(selectcolor)

    label $w.options.title2 -text "Page Size:"
    radiobutton $w.options.fixed -text "Fixed in Size" \
	-variable page(scale) \
	-value fixed -selectcolor $menu(selectcolor) \
	-command "PageDialogSize $w"
    radiobutton $w.options.scaled -text "Scale to fit Page" \
	-variable page(scale) \
	-value scaled -selectcolor $menu(selectcolor) \
	-command "PageDialogSize $w"

    frame $w.options.s -relief groove -borderwidth 2
    radiobutton $w.options.s.letter -text "Letter(8.5 x 11 in)" \
	-variable page(size) -value letter -selectcolor $menu(selectcolor)
    radiobutton $w.options.s.legal -text "Legal(8.5 x 14 in)" \
	-variable page(size) \
	-value legal -selectcolor $menu(selectcolor)
    radiobutton $w.options.s.tabloid -text "Tabloid(11 x 17 in)" \
	-variable page(size) -value tabloid -selectcolor $menu(selectcolor)
    radiobutton $w.options.s.poster -text "Poster(36 x 48 in)" \
	-variable page(size) -value poster -selectcolor $menu(selectcolor)
    radiobutton $w.options.s.a4 -text "A4(210 x 297 mm)" \
	-variable page(size) \
	-value a4 -selectcolor $menu(selectcolor)
    radiobutton $w.options.s.other -text "Other (inches)" \
	-variable page(size) \
	-value other -selectcolor $menu(selectcolor)
    radiobutton $w.options.s.othermm -text "Other (mm)" \
	-variable page(size) \
	-value othermm -selectcolor $menu(selectcolor)

    frame $w.options.s.wh
    label $w.options.s.wh.title3 -text "Width"
    entry $w.options.s.wh.width -textvariable page(width) \
	-width 10 -relief sunken -bd 2

    label $w.options.s.wh.title4 -text "Height"
    entry $w.options.s.wh.height -textvariable page(height) \
	-width 10 -relief sunken -bd 2

    grid rowconfigure $w.options 0 -pad 4
    grid rowconfigure $w.options 1 -pad 4
    grid rowconfigure $w.options 2 -pad 4

    grid $w.options.title1 -row 0 -column 0 -padx 4 -sticky w
    grid $w.options.portrait -row 0 -column 1 -padx 4 -sticky w
    grid $w.options.landscape -row 0 -column 2 -padx 4 -sticky w

    grid $w.options.title2 -row 1 -column 0 -padx 4 -sticky w
    grid $w.options.scaled -row 1 -column 1 -padx 4 -sticky w
    grid $w.options.fixed -row 1 -column 2 -padx 4 -sticky w

    grid $w.options.s -row 2 -column 1 -padx 4 -sticky w -columnspan 2

    grid rowconfigure $w.options.s 0 -pad 4
    grid rowconfigure $w.options.s 1 -pad 4
    grid rowconfigure $w.options.s 2 -pad 4
    grid rowconfigure $w.options.s 3 -pad 4
    grid rowconfigure $w.options.s 4 -pad 4

    grid $w.options.s.letter -row 0 -column 1 -padx 4 -sticky w
    grid $w.options.s.legal -row 1 -column 1 -padx 4 -sticky w
    grid $w.options.s.tabloid -row 2 -column 1 -padx 4 -sticky w
    grid $w.options.s.poster -row 3 -column 1 -padx 4 -sticky w
    grid $w.options.s.a4 -row 4 -column 1 -padx 4 -sticky w
    grid $w.options.s.other -row 5 -column 1 -padx 4 -sticky w
    grid $w.options.s.othermm -row 6 -column 1 -padx 4 -sticky w

    grid $w.options.s.wh -row 5 -column 2 -padx 4 -sticky w -rowspan 2

    grid rowconfigure $w.options.s.wh 0 -pad 4
    grid rowconfigure $w.options.s.wh 1 -pad 4

    grid $w.options.s.wh.title3 -row 0 -column 0 -padx 4 -sticky w
    grid $w.options.s.wh.width -row 0 -column 1 -padx 4 -sticky w
    grid $w.options.s.wh.title4 -row 1 -column 0 -padx 4 -sticky w
    grid $w.options.s.wh.height -row 1 -column 1 -padx 4 -sticky w

    # Buttons

    button $w.buttons.ok -text "OK" -default active \
	-command {set page(ok) 1}
    button $w.buttons.cancel -text "Cancel" -command {set page(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set page(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    PageDialogSize $w
    DialogCenter $w
    DialogWait $w page(ok) $w.buttons.ok 
    DialogDismiss $w

    if {$page(ok)} {
	set ps(orient) $page(orient)
	set ps(scale) $page(scale)
	set ps(size) $page(size)
	set ps(width) $page(width)
	set ps(height) $page(height)
    }
    return $page(ok)
}

proc PageDialogSize {w} {
    global page

    if {$page(scale) == "scaled"} {
	$w.options.s.letter configure -state normal
	$w.options.s.legal configure -state normal
	$w.options.s.tabloid configure -state normal
	$w.options.s.poster configure -state normal
	$w.options.s.a4 configure -state normal
	$w.options.s.other configure -state normal
	$w.options.s.othermm configure -state normal
    } else {
	$w.options.s.letter configure -state disabled
	$w.options.s.legal configure -state disabled
	$w.options.s.tabloid configure -state disabled
	$w.options.s.poster configure -state disabled
	$w.options.s.a4 configure -state disabled
	$w.options.s.other configure -state disabled
	$w.options.s.othermm configure -state disabled
    }
}






