/*
 *      Copyrights:
 *
 *    	  Copyright (c) 2000 Smithsonian Astrophysical Observatory
 *
 *        Permission to use, copy, modify, distribute, and  sell  this
 *        software  and  its  documentation  for any purpose is hereby
 *        granted without  fee,  provided  that  the  above  copyright
 *        notice  appear  in  all  copies and that both that copyright
 *        notice and this permission notice appear in supporting docu-
 *        mentation,  and  that  the  name  of the  Smithsonian Astro-
 *	  physical Observatory not be used in advertising or publicity
 *  	  pertaining to distribution of the software without specific,
 *	  written  prior  permission.   The Smithsonian  Astrophysical 
 *        Observatory makes no representations about  the  suitability
 *	  of  this  software for any purpose.  It is provided  "as is"
 *	  without express or implied warranty.
 *        THE  SMITHSONIAN  ASTROPHYSICAL  OBSERVATORY  DISCLAIMS  ALL
 *	  WARRANTIES  WITH  REGARD  TO  THIS  SOFTWARE,  INCLUDING ALL
 *	  IMPLIED  WARRANTIES  OF  MERCHANTABILITY AND FITNESS, IN  NO
 *	  EVENT SHALL THE  SMITHSONIAN  ASTROPHYSICAL  OBSERVATORY  BE 
 *	  LIABLE FOR  ANY SPECIAL, INDIRECT  OR  CONSEQUENTIAL DAMAGES
 *	  OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS  OF USE,  DATA
 *	  OR  PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 *	  OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION  WITH
 *	  THE  USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#include <tl.h>

#ifdef ANSI_FUNC
double
tlp2i (double dp, double tlmin, int type)
#else
double tlp2i (dp, tlmin, type)
     double dp;
     double tlmin;
     int type;
#endif
{
  switch(type){
  case 'B':
  case 'I':
  case 'U':
  case 'J':
  case 'V':
    return(dp - tlmin + 1);
  case 'E':
  case 'D':
    return(dp - tlmin + 0.5);
  default:
    return(dp - tlmin + 1);
  }
}

#ifdef ANSI_FUNC
int
itlp2i (double dp, double tlmin, int type)
#else
int itlp2i (dp, tlmin, type)
     double dp;
     double tlmin;
     int type;
#endif
{
  /* NB: all cases are the same here, since in the case of continuous
     floats, we have to add an extra 0.5 before integerizing */
  return((int)(dp - tlmin + 1));
}

#ifdef ANSI_FUNC
double
tli2p (double di, double tlmin, int type)
#else
double tli2p (di, tlmin, type)
     double di;
     double tlmin;
     int type;
#endif
{
  switch(type){
  case 'B':
  case 'I':
  case 'U':
  case 'J':
  case 'V':
    return(di + tlmin - 1);
  case 'E':
  case 'D':
    return(di + tlmin - 0.5);
  default:
    return(di + tlmin - 1);
  }
}

#ifdef ANSI_FUNC
double
tldim (double tlmax, double tlmin, int type)
#else
double tldim (tlmax, tlmin, type)
     double tlmax;
     double tlmin;
     int type;
#endif
{
  switch(type){
  case 'B':
  case 'I':
  case 'U':
  case 'J':
  case 'V':
    return(tlmax-tlmin+1);
  case 'E':
  case 'D':
    return(tlmax-tlmin);
  default:
    return(tlmax-tlmin+1);
  }
}

