#include <string.h>

#include <strstream.h>
#include <iomanip.h>

#include "card.h"

FitsCard::FitsCard()
{
  card_ = new char[FTY_CARDLEN];
  managed = 1;
}

FitsCard::FitsCard(char* c)
{
  card_ = c;
  managed = 0;
}

FitsCard::FitsCard(const FitsCard& a)
{
  card_ = new char[FTY_CARDLEN];
  managed = 1;
  memcpy(card_, a.card_, FTY_CARDLEN);
}

FitsCard::~FitsCard()
{
  if (managed)
    delete [] card_;
}

FitsCard& FitsCard::operator=(const FitsCard& a)
{
  memcpy(card_, a.card_, FTY_CARDLEN);
  return *this;
}

FitsCard& FitsCard::clear()
{
  memset(card_, ' ', FTY_CARDLEN);
  return *this;
}

FitsCard& FitsCard::setKey(const char* name)
{
  if (name) {
    memset(card_, ' ', 8);
    ostrstream str(card_, 8);
    str << name;
  }

  return *this;
}

FitsCard& FitsCard::setLogical(const char* name, int value, 
			       const char* comment)
{
  setKey(name);

  memset(card_+8, ' ', FTY_CARDLEN-8);
  ostrstream str(card_+8, FTY_CARDLEN-8);
  str << "=                    " << (value ? 'T' : 'F');
  if (comment)
    str << " / " << comment;

  return *this;
}

FitsCard& FitsCard::setInteger(const char* name, int value, 
			       const char* comment)
{
  setKey(name);

  memset(card_+8, ' ', FTY_CARDLEN-8);
  ostrstream str(card_+8, FTY_CARDLEN-8);
  str << "= " << setw(20) << value;
  if (comment)
    str << " / " << comment;

  return *this;
}

FitsCard& FitsCard::setReal(const char* name, double value, 
			    int prec, const char* comment)
{
  setKey(name);

  memset(card_+8, ' ', FTY_CARDLEN-8);
  ostrstream str(card_+8, FTY_CARDLEN-8);
  str << "= " << setw(20) << setprecision(prec) << value;
  if (comment)
    str << " / " << comment;

  return *this;
}

FitsCard& FitsCard::setComplex(const char* name, double real, double img, 
			int prec, const char* comment)
{
  setKey(name);

  memset(card_+8, ' ', FTY_CARDLEN-8);
  ostrstream str(card_+8, FTY_CARDLEN-8);
  str << "= " << setw(20) << setprecision(prec) << real << img;
  if (comment)
    str << " / " << comment;

  return *this;
}

FitsCard& FitsCard::setString(const char* name, const char* value, 
			      const char* comment)
{
  setKey(name);

  memset(card_+8, ' ', FTY_CARDLEN-8);
  ostrstream str(card_+8, FTY_CARDLEN-8);
  str << "= '" << value << '\'';
  if (comment)
    str << " / " << comment;

  return *this;
}

FitsCard& FitsCard::setComment(const char* name, const char* value)
{
  setKey(name);

  memset(card_+8, ' ', FTY_CARDLEN-8);
  ostrstream str(card_+8, FTY_CARDLEN-8);
  str << "= " << value;

  return *this;
}

int FitsCard::getLogical()
{
  for (int i=10; i<80; i++)
    if (card_[i] != ' ')
      return (card_[i] == 'T' || card_[i] == 't');

  return 0;
}

int FitsCard::getInteger()
{
  istrstream str(card_+10, FTY_CARDLEN-10);
  int r;
  str >> r;
  return r;
}

double FitsCard::getReal()
{
  istrstream str(card_+10, FTY_CARDLEN-10);
  double r;
  str >> r;
  return r;
}

void FitsCard::getComplex(double* r, double* i)
{
  istrstream str(card_+10, FTY_CARDLEN-10);
  str >> *r >> *i;
}

char* FitsCard::getString()
{
  char value[FTY_CARDLEN];
  value[0] = '\0';

  int i,j;
  // find first '
  for (i=10; i<FTY_CARDLEN; i++)
    if (card_[i] == '\'')
      break;

  // now find last '
  i++;
  for (j=0; i<FTY_CARDLEN; i++,j++) {
    if (card_[i] == '\'')
      if (i+1 == FTY_CARDLEN || card_[i+1] != '\'') {
	break;
      }
      else
	i++;

    value[j] = card_[i];
  }

  // strip any spaces
  for (; j && value[j-1]==' '; j--);
  value[j] = '\0';

  // return result
  char* cpy = new char[strlen(value)+1];
  strcpy(cpy, value);
  return cpy;
}

char* FitsCard::getComment()
{
  char* cpy = new char[FTY_CARDLEN-7];
  memcpy(cpy, card_+8, FTY_CARDLEN-8);
  cpy[FTY_CARDLEN-8] = '\0';

  return cpy;
}

