#include "baseellipse.h"
#include "frame.h"
#include "util.h"

BaseEllipse::BaseEllipse(Frame* p, const Vector& ctr, double a, 
			 const char* clr,  const char* f, const char* t, 
			 unsigned short prop)
  : BaseMarker(p, ctr, 48, clr, f, t, prop) 
{
  radii = NULL;
  angle = a;

  if (!parent->isIIS())
    flip = FlipY();
}

BaseEllipse::~BaseEllipse()
{
  if (radii)
    delete [] radii;
}

void BaseEllipse::updateCoords(Matrix& m)
{
  for (int i=0; i<annuli; i++)
    radii[i] *= Scale(m);
  Marker::updateCoords(m);
}

void BaseEllipse::rotate(const Vector& v, int h)
{
  // v is in ref coords
  // handles are in canvas coords

  Vector a = v - center;
  Vector b = (handle[h-1] * parent->canvasToRef) - center;

  if (parent->isIIS())
    angle -= a.angle() - b.angle();
  else
    angle += a.angle() - b.angle();

  updateBBox();
  doCallBacks(&rotateCB);
}

int BaseEllipse::isIn(const Vector& v)
{
  // v is in canvas coords

  Vector r = radii[annuli-1];
  if (properties & FIXED)
    r /= parent->getZoom();

  Vector p = v * 
    parent->canvasToRef * Translate(-center) * flip * Rotate(-angle);

  if ((p[0]*p[0])/(r[0]*r[0]) + (p[1]*p[1])/(r[1]*r[1]) <= 1)
    return 1;
  else
    return 0;
}

void BaseEllipse::setAngle(double a)
{
  angle = a;
  updateBBox();

  doCallBacks(&rotateCB);
}

// private

void BaseEllipse::calcPoints(double zoom)
{
  Matrix m;
  if (properties & FIXED)
    m = Scale(1/zoom) * Rotate(angle) * flip * Translate(center);
  else
    m = Rotate(angle) * flip * Translate(center);

  for (int i=0; i<annuli; i++) {
    for (int j=0; j<segments; j++) {
      double theta = 2*M_PI / segments * j;
      vertices[i][j] = 
	Vector(radii[i][0]*cos(theta), radii[i][1]*sin(theta)) * m;
    }
  }
}
