#include <stdlib.h>
#include <string.h>

#include "colorscaletrue16.h"
#include "util.h"

ColorScaleTrueColor16::ColorScaleTrueColor16(int s, Visual* v) : ColorScale(s)
{
  colors_ = new unsigned char[s*2];

  int rs,gs,bs;
#ifndef _WIN32
  unsigned short rm = decodeMask((unsigned short)v->red_mask, &rs);
  unsigned short gm = decodeMask((unsigned short)v->green_mask, &gs);
  unsigned short bm = decodeMask((unsigned short)v->blue_mask, &bs);
#else
  unsigned short rm = decodeMask((unsigned short)0x7C00, &rs);
  unsigned short gm = decodeMask((unsigned short)0x03E0, &gs);
  unsigned short bm = decodeMask((unsigned short)0x001F, &bs);
#endif

  for (int i=0; i<s; i++) {
    unsigned short r = psColors_[i*3+2];
    unsigned short g = psColors_[i*3+1];
    unsigned short b = psColors_[i*3];
    unsigned short result = 0;
    result |= rs>0 ? ((r & rm) << rs) : ((r & rm) >> -rs);
    result |= gs>0 ? ((g & gm) << gs) : ((g & gm) >> -gs);
    result |= bs>0 ? ((b & bm) << bs) : ((b & bm) >> -bs);

    memcpy(colors_+i*2, &result, 2);
  }
}

ColorScaleTrueColor16::~ColorScaleTrueColor16()
{
  if (colors_)
    delete [] colors_;
}

LinearScaleTrueColor16::LinearScaleTrueColor16(int s, 
					       unsigned short* indexCells,
					       unsigned char* colorCells,
					       int count, Visual* v)
  : LinearScale(s, indexCells, colorCells, count), 
    ColorScaleTrueColor16(s, v),
    ColorScale(s) {}

LogScaleTrueColor16::LogScaleTrueColor16(int s,
					 unsigned short* indexCells,
					 unsigned char* colorCells,
					 int count, Visual* v)
  : LogScale(s, indexCells, colorCells, count), 
    ColorScaleTrueColor16(s, v),
    ColorScale(s) {}

SquaredScaleTrueColor16::SquaredScaleTrueColor16(int s,
						 unsigned short* indexCells,
						 unsigned char* colorCells,
						 int count, Visual* v)
  : SquaredScale(s,indexCells,colorCells,count), 
    ColorScaleTrueColor16(s,v), 
    ColorScale(s) {}

SqrtScaleTrueColor16::SqrtScaleTrueColor16(int s,
					   unsigned short* indexCells,
					   unsigned char* colorCells,
					   int count, Visual* v)
  : SqrtScale(s, indexCells, colorCells, count), 
    ColorScaleTrueColor16(s, v),
    ColorScale(s) {}

IISScaleTrueColor16::IISScaleTrueColor16(unsigned short* indexCells,
					 unsigned char* colorCells,
					 int count, Visual* v)
  : IISScale(indexCells, colorCells, count),
    ColorScaleTrueColor16(IISSIZE, v),
    ColorScale(IISSIZE) {}
