#include <math.h>
#include <iostream.h>

#include "contourscale.h"
#include "frame.h"

ContourScale::ContourScale(int s) : size_(s)
{
  level_ = new float[size_+1];

  for (int i=0; i<=size_; i++)
    level_[i] = 0;
}

ContourScale::ContourScale(int s, float* l) : size_(s)
{
  level_ = new float[size_+1];

  for (int i=0; i<=size_; i++)
    level_[i] = l[i];
}

ContourScale::~ContourScale()
{
  if (level_)
    delete [] level_;
}

ostream& operator<<(ostream& s, const ContourScale& c)
{
  for (int i=0; i<c.size_+1; i++)
    s << c.level_[i] << endl;
  return s;
}

// Note: this an inverse algorithm. We have y, we need x

LinearContourScale::LinearContourScale(int s, float low, float high) 
  : ContourScale(s)
{
  float diff = high-low;
  for (int i=0; i<=size_; i++)
    level_[i] = (float)i/size_*diff + low;
}

LogContourScale::LogContourScale(int s, float low, float high) 
  : ContourScale(s)
{
  float diff = high-low;
  for (int i=0; i<=size_; i++) {
    if (i>0) {
      float y = (float)i/size_;
      float x = pow(10,(y*log10(SCALESIZE)))/SCALESIZE;
      level_[i] = x*diff+low;
    }
    else
      level_[i] = low;
  }
}

SquaredContourScale::SquaredContourScale(int s, float low, float high)
  : ContourScale(s)
{
  float diff = high-low;
  for (int i=0; i<=size_; i++)
    level_[i] = sqrt((float)i/size_)*diff+low;
}

SqrtContourScale::SqrtContourScale(int s, float low, float high) 
  : ContourScale(s)
{
  float diff = high-low;
  for (int i=0; i<=size_; i++) {
    float y = (float)i/size_;
    level_[i] = (y*y)*diff+low;
  }
}

