/*
 *
 * ctest -- client test for xpa
 *
 */

/*
 *      Copyrights:
 *
 *    	  Copyright (c) 1996, 1998 Smithsonian Astrophysical Observatory
 *
 *        Permission to use, copy, modify, distribute, and  sell  this
 *        software  and  its  documentation  for any purpose is hereby
 *        granted without  fee,  provided  that  the  above  copyright
 *        notice  appear  in  all  copies and that both that copyright
 *        notice and this permission notice appear in supporting docu-
 *        mentation,  and  that  the  name  of the  Smithsonian Astro-
 *	  physical Observatory not be used in advertising or publicity
 *  	  pertaining to distribution of the software without specific,
 *	  written  prior  permission.   The Smithsonian  Astrophysical 
 *        Observatory makes no representations about  the  suitability
 *	  of  this  software for any purpose.  It is provided  "as is"
 *	  without express or implied warranty.
 *        THE  SMITHSONIAN  ASTROPHYSICAL  OBSERVATORY  DISCLAIMS  ALL
 *	  WARRANTIES  WITH  REGARD  TO  THIS  SOFTWARE,  INCLUDING ALL
 *	  IMPLIED  WARRANTIES  OF  MERCHANTABILITY AND FITNESS, IN  NO
 *	  EVENT SHALL THE  SMITHSONIAN  ASTROPHYSICAL  OBSERVATORY  BE 
 *	  LIABLE FOR  ANY SPECIAL, INDIRECT  OR  CONSEQUENTIAL DAMAGES
 *	  OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS  OF USE,  DATA
 *	  OR  PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 *	  OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION  WITH
 *	  THE  USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#include <xpap.h>

extern char *optarg;
extern int optind;

int  quiet=0;
int  dowait=0;
int  n=0;
int  save_bytes=-1;
int  exitonerror=0;
char *save_buf=NULL;
char *mode="";
char name[SZ_LINE];

#define MAX_FPS 4

#ifdef ANSI_FUNC
int 
send_cb (void *client_data, void *call_data, char *paramlist,
	 char **buf, int *len)
#else
int send_cb(client_data, call_data, paramlist, buf, len)
     void *client_data;
     void *call_data;
     char *paramlist;
     char **buf;
     int *len;
#endif
{
  char *s = (char *)client_data;
  XPA xpa = (XPA)call_data;
  char tbuf[SZ_LINE];
  int sendbuf=0;

  /* introduce ourselves */
  if( !quiet ){
    fprintf(stderr, "SEND_CB #%d: %s:%s (%s)\n",
	    n++, xpa_class(xpa), xpa_name(xpa), s);
  }

  /* process special paramlist tokens */
  if( paramlist && *paramlist ){
    if( !quiet )
      fprintf(stderr, "\tparamlist: %s\n", paramlist);
    if( !strncmp(paramlist, "buf", 3) ){
      sendbuf=1;
    }
    else if( !strncmp(paramlist, "error", 5) ){
      if( !quiet )
	fprintf(stderr, "\treturning error: %s\n", &paramlist[6]);
      *len = 0;
      *buf = NULL;
      XPAError(xpa, &paramlist[6]);
      return(-1);
    }
    else if( !strcmp(paramlist, "exit") ){
      if( !quiet )
	fprintf(stderr, "Exiting\n");
      exit(0);
    }
    else if( !strcmp(paramlist, "wait") ){
      fprintf(stderr, "Press <CR> to continue ...");
      gets(tbuf);
    }
  }
  else if( dowait ){
    fprintf(stderr, "Press <CR> to continue ...");
    gets(tbuf);
  }

  /* return information about this xpa */
  if( !sendbuf ){
    sprintf(tbuf,
	  "nclass: %s\nname: %s\nmethod: %s\nsendian: %s\ncendian: %s\n",
	  xpa_class(xpa), xpa_name(xpa), xpa_method(xpa),
	  xpa_sendian(xpa), xpa_cendian(xpa));
  
    if( (xpa->send_mode & XPA_MODE_FILLBUF) ){
      send(xpa_datafd(xpa), tbuf, strlen(tbuf), 0);
      *len = 0;
      *buf = NULL;
      if( !quiet)
	fprintf(stderr, "\tcallback writes %d bytes to client\n",
		strlen(tbuf));
    }
    /* return the buffer and let xpa transmit it */
    else{
      *len = strlen(tbuf);
      *buf = (char *)malloc(*len);
      memcpy(*buf, tbuf, *len);
      if( !quiet)
	fprintf(stderr, "\tcallback returns %d bytes to xpa handler\n",
		strlen(tbuf));
    }
  }
  /* return the last buffer we were sent */
  else{
    if( (xpa->send_mode & XPA_MODE_FILLBUF) ){
      send(xpa_datafd(xpa), save_buf, save_bytes, 0);
      *len = 0;
      *buf = NULL;
      if( !quiet)
	fprintf(stderr, "\tcallback writes %d bytes to client\n", save_bytes);
    }
    /* return the buffer and let xpa transmit it */
    else{
      *len = save_bytes;
      *buf = (char *)malloc(*len);
      memcpy(*buf, save_buf, *len);
      if( !quiet)
	fprintf(stderr, "\tcallback returns %d bytes to xpa handler\n",
		save_bytes);
    }
  }
  return(0);
}

#ifdef ANSI_FUNC
int 
receive_cb (void *client_data, void *call_data, char *paramlist,
	    char *buf, int len)
#else
int receive_cb(client_data, call_data, paramlist, buf, len)
     void *client_data;
     void *call_data;
     char *paramlist;
     char *buf;
     int len;
#endif
{
  XPA xpa = (XPA)call_data;
  char *s = (char *)client_data;
  char tbuf[SZ_LINE];
  char cbuf[SZ_LINE];
  char *errs[1];
  int i;
  int ip;
  int got;
  int xwait;

  if( !quiet ){
    fprintf(stderr, "RECEIVE_CB #%d: %s:%s (%s)\n",
	    n++, xpa_class(xpa), xpa_name(xpa), s);
  }
  if( !quiet ){
    fprintf(stderr, "\tbuf: %d bytes\n", len);
  }

  /* process param list */
  if( paramlist && *paramlist ){
    if( !quiet )
      fprintf(stderr, "\tparamlist: %s\n", paramlist);
    if( !strcmp(paramlist, "free") ){
      if( !quiet )
	fprintf(stderr, "Freeing xpa struct\n");
      XPAFree(xpa);
      return(0);
    }
    else if( !strncmp(paramlist, "error", 5) ){
      if( !quiet )
	fprintf(stderr, "Processing error: %s\n", &paramlist[6]);
      XPAError(xpa, &paramlist[6]);
      return(-1);
    }
    else if( !strcmp(paramlist, "exit") ){
      if( !quiet )
	fprintf(stderr, "Exiting\n");
      exit(0);
    }
    else if( !strcmp(paramlist, "wait") ){
      fprintf(stderr, "Press <CR> to continue ...");
      gets(tbuf);
    }
    else if( !strncmp(paramlist, "xpaset", 6) ){
      ip = 0;
      word(paramlist, tbuf, &ip);
      if( word(paramlist, tbuf, &ip) ){
	if( !quiet )
	  fprintf(stderr, "calling XPASet(%s, \"%s\")\n",
		  tbuf, &(paramlist[ip]));
	got = XPASet(NULL, tbuf, &(paramlist[ip]), mode,
		     paramlist, strlen(paramlist), NULL, errs, 1);
	if( got == 0 ){
	  if( !quiet )
	    fprintf(stderr, "no XPA access points matched template %s\n",
		    tbuf);
	}
	else if( errs[0] != NULL ){
	  if( !quiet )
	    fprintf(stderr, "Error on xpaset to %s: %s\n", tbuf, errs[0]);
	  free(errs[0]);
	  if( exitonerror )
	    exit(1);
	}
	else{
	  if( !quiet )
	    fprintf(stderr, "XPASet to %s successful\n", tbuf);
	}
      }
      return(0);
    }
  }
  else if( dowait ){
    fprintf(stderr, "Press <CR> to continue ...");
    gets(tbuf);
  }

  /* reset save buffer */
  if( save_buf != NULL ){
    free(save_buf);
    save_buf = NULL;
  }
  save_bytes = 0;

  xwait = dowait;
  if( !(xpa->receive_mode & XPA_MODE_FILLBUF) ){
    while( (got=recv(xpa_datafd(xpa), tbuf, SZ_LINE, 0)) >0 ){
      if( xwait >0 ){
	fprintf(stderr, "got %d bytes ... press <CR> to continue ...", got);
	gets(cbuf);
	xwait--;
      }
      i = save_bytes;
      save_bytes += got;
      if( save_buf == NULL )
	save_buf = (char *)malloc(save_bytes);
      else
	save_buf = (char *)realloc(save_buf, save_bytes);
      memcpy(&save_buf[i], tbuf, got);
    }
    if( !quiet )
      fprintf(stderr, "callback read %d bytes\n", save_bytes);
  }
  else{
    save_bytes = len;
    save_buf = (char *)malloc(len);
    memcpy(save_buf, buf, len);
  }
  return(0);
}

#ifdef ANSI_FUNC
int 
info_cb (void *client_data, void *call_data, char *paramlist)
#else
int info_cb(client_data, call_data, paramlist)
     void *client_data;
     void *call_data;
     char *paramlist;
#endif
{
  XPA xpa = (XPA)call_data;
  char *s = (char *)client_data;
  char tbuf[SZ_LINE];
  char xtemplate[SZ_LINE];
  FILE *fd;
  char *names[MAX_FPS];
  char *bufs[MAX_FPS];
  char *errs[MAX_FPS];
  int lens[MAX_FPS];
  int i;
  int ip;
  int got;

  if( !quiet ){
    fprintf(stderr, "INFO_CB #%d: %s:%s (%s)\n",
	    n++, xpa_class(xpa), xpa_name(xpa), s);
  }

  if( paramlist && *paramlist ){
    if( !quiet )
      fprintf(stderr, "\tparamlist: %s\n", paramlist);
    ip = 0;
    word(paramlist, xtemplate, &ip);
    if( !strcmp(xtemplate, "xpaget") ){
      word(paramlist, xtemplate, &ip);
      got = XPAGet(xpa, xtemplate, &(paramlist[ip]), NULL,
		   bufs, lens, names, errs, MAX_FPS);
      if( !quiet )
	fprintf(stderr, "XPAGet (%s) returned %d buffers\n", xtemplate, got);
      for(i=0; i<got; i++){
	if( !quiet )
	  fprintf(stderr, "\t%d: %s\n", i, names[i]);
	if( errs[i] == NULL ){
	  if( lens[i] > 0 ){
	    write(fileno(stderr), bufs[i], lens[i]);
	  }
	  if( !quiet )
	    fprintf(stderr, "\n");
	}
	else{
	  write(fileno(stderr), errs[i], strlen(errs[i]));
	  if( exitonerror )
	    exit(1);
	}
	if( bufs[i] )
	  free(bufs[i]);
	if( names[i] )
	  free(names[i]);
	if( errs[i] )
	  free(errs[i]);
      }
    }
  }
  return(0);
}

#ifdef ANSI_FUNC
int 
main (int argc, char **argv)
#else
main(argc, argv)
     int argc;
     char **argv;
#endif
{
  char *paramlist=NULL;
  char *xtemplate="*:*";
  char tbuf[SZ_LINE];
  char name[SZ_LINE];
  char cmd[SZ_LINE];
  char fxpa[SZ_LINE];
  char *names[MAX_FPS];
  char *bufs[MAX_FPS];
  char *errs[MAX_FPS];
  char *buf=NULL;
  char *file;
  int c;
  int lens[MAX_FPS];
  int len;
  int i;
  int got;
  int maxbufs;
  int j=0;
  int loop=1;
  int delay=0;
  int get=1;
  int set=0;
  int info=0;
  int pipe=0;
  int doxpa = 0;
  int xpaopen=0;
  int usebuf=1;
  int wait=0;
  XPA xpa=NULL;
  XPA xpa1=NULL;
  int fds[1];

  *cmd = '\0';
  /* process switch arguments */
  while ((c = getopt(argc, argv, "d:ef:gil:opqsuwx:")) != -1){
    switch(c){
    case 'd':
      delay = atoi(optarg);
      break;
    case 'e':
      exitonerror++;
      break;
    case 'f':
      sprintf(cmd, "stest %s &\n", optarg);
      sprintf(fxpa, "%s1", optarg);
      break;
    case 'g':
      get = 1;
      set = 0;
      info = 0;
      break;
    case 'i':
      get = 0;
      set = 0;
      info = 1;
      break;
    case 'l':
      loop = atoi(optarg);
      break;
    case 'o':
      xpaopen = 1;
      break;
    case 'p':
      pipe = 1;
      fds[0] = fileno(stdout);
      break;
    case 'q':
      quiet = 1;
      break;
    case 's':
      get = 0;
      set = 1;
      info = 0;
      break;
    case 'u':
      usebuf = 0;
      break;
    case 'w':
      wait = 1;
      break;
    case 'x':
      doxpa = 1;
      strcpy(name, optarg);
      break;
    }
  }
  if( optind < argc ){
    xtemplate = argv[optind];
    optind++;
  }

  /* make up the paramlist */
  paramlist = XPAArgvParamlist(argc, argv, optind);

  /* for setting, make up a number of bufs that we will send */
  if( set ){
    /* must be less than MAX_FPS */
    maxbufs = 2;

    bufs[0] = (char *)malloc(SZ_LINE);
    strcpy(bufs[0], "this is a short string");
    lens[0] = strlen(bufs[0]);

    lens[1] = 256*1000+1;
    bufs[1] = (char *)malloc(lens[1]);
    for(j=0, buf=bufs[1]; j<lens[1]; j++){
      buf[j] = j%256;
    }
  }

  /* background process initialization */
  if( *cmd != '\0' ){
    fprintf(stderr, "starting bkgd process: %s", cmd);
    system(cmd);
    for(i=0; i<5; i++){
      if( XPAAccess(fxpa, "g") ){
	fprintf(stderr, "process is started\n");
	break;
      }
      else
	sleep(1);
    }
  }

  /* start up an xpa server, if necessary */
  if( doxpa ) {
    sprintf(tbuf, "%s from ctest", name);
    if( (xpa1 = XPANew("CLIENT", name, name,
		       send_cb, tbuf, mode, receive_cb, tbuf, mode)) == NULL ){
      fprintf(stderr, "ERROR: could not init client xpa: %s\n", name);
    }
    else{
      fprintf(stderr, "%s using method: %s\n",
	      xpa_name(xpa1), xpa_method(xpa1));
    }
  }

  /* XPAOpen */
  if( xpaopen ){
    if( (xpa = XPAOpen(NULL)) == NULL ){
      fprintf(stderr, "ERROR: could not open any xpa access points: %s\n",
	      xtemplate);
      exit(1);
    }
  }

  /* iterate */
  for(j=0; j<loop; j++){
    if( loop > 1 ){
      if( !quiet ){
	fprintf(stderr, "CLIENT #%d: %s\n", j, xtemplate);
	if( paramlist && *paramlist )
	  fprintf(stderr, "\tparamlist: %s\n", paramlist);
      }
    }
    /* XPAGet, XPAGetFd */
    if( get ){
      if( pipe ){
	got = XPAGetFd(xpa, xtemplate, paramlist, mode,
		       fds, names, errs, -MAX_FPS);
      }
      else{
	got = XPAGet(xpa, xtemplate, paramlist, mode, 
		     bufs, lens, names, errs, MAX_FPS);
	for(i=0; i<got; i++){
	  if( !quiet ){
	    fprintf(stderr, "\treceived from: %s\n", names[i]);
	    if( errs[i] == NULL ){
	      if( lens[i] > 0 ){
		write(fileno(stdout), bufs[i], lens[i]);
	      }
	      fprintf(stderr, "\n");
	    }
	    else{
	      write(fileno(stderr), errs[i], strlen(errs[i]));
	      if( exitonerror )
		exit(1);
	    }
	  }
	  if( bufs[i] )
	    free(bufs[i]);
	  if( names[i] )
	    free(names[i]);
	  if( errs[i] )
	    free(errs[i]);
	}
      }
    }
    
    /* XPASet, XPASetFd */
    else if( set ){
      if( pipe ){
	if( !quiet )
	  fprintf(stderr, "\tsending data from stdin\n");
	got =XPASetFd(xpa, xtemplate, paramlist, mode, 
		      fileno(stdin), names, errs, MAX_FPS);
      }
      else{
	if( !quiet )
	  fprintf(stderr, "\tsending %d bytes\n", lens[j%maxbufs]);
	got = XPASet(xpa, xtemplate, paramlist, mode,
		     bufs[j%maxbufs], lens[j%maxbufs],
		     names, errs, MAX_FPS);
      }
      for(i=0; i<got; i++){
	if( errs[i] != NULL ){
	  fprintf(stderr, "Error from server: %s\n", names[i]);
	  write(fileno(stderr), errs[i], strlen(errs[i]));
	  if( exitonerror )
	    exit(1);
	}
	if( names[i] )
	  free(names[i]);
	if( errs[i] )
	  free(errs[i]);
      }
    }

    /* XPAInfo */
    else if( info ){
      got = XPAInfo(xpa, xtemplate, paramlist, mode, NULL, NULL, 0);
    }

    /* if we processed no access points, let`s hear about it */
    if( got == 0 ){
      fprintf(stderr, "XPA$ERROR: no xpa access points processed\n");
    }
    else{
	fprintf(stderr, "CLIENT #%d complete\n", j);
    }

    /* wait for next iteration */
    if( wait ){
      fprintf(stderr, "Press <CR> to continue ...");
      gets(tbuf);
    }

    /* delay */
    if( delay )
      sleep(delay);
  }

  /* XPAOpen/XPAClose */
  if( xpaopen ){
    XPAClose(xpa);
  }

  /* free up space */
  if( set ){
    for(j=0; j<maxbufs; j++){
      if( bufs[j] )
	free(bufs[j]);
    }
  }
  if(  paramlist && *paramlist )
    free(paramlist);
  exit(0);
}


